@@ -51,6 +51,9 @@ public:
         const std::shared_ptr<ScriptingRuntime>& runtime);
     ~JSLayout() override;
 
+    JSLayout(const JSLayout&) = delete;
+    JSLayout& operator=(const JSLayout&) = delete;
+
     void processCdsObject(std::shared_ptr<CdsObject> obj, fs::path rootpath) override;
 };
 
