@@ -48,7 +48,7 @@ public:
         std::shared_ptr<Server> server,
         std::shared_ptr<web::SessionManager> session_manager);
 
-    virtual ~Context();
+    virtual ~Context() = default;
 
     std::shared_ptr<Config> getConfig() const
     {
