@@ -35,9 +35,9 @@
 #include <cstdlib>
 
 #include <netinet/in.h>
-#include <zlib.h>
 
 #include "config/config_manager.h"
+#include "util/thread_runner.h"
 #include "util/tools.h"
 
 //#define MYSQL_SET_NAMES "/*!40101 SET NAMES utf8 */"
@@ -50,18 +50,15 @@
 #define MYSQL_UPDATE_1_2_3 "ALTER TABLE `mt_cds_object` CHANGE `auxdata` `auxdata` BLOB NULL DEFAULT NULL"
 #define MYSQL_UPDATE_1_2_4 "ALTER TABLE `mt_cds_object` CHANGE `resources` `resources` BLOB NULL DEFAULT NULL"
 #define MYSQL_UPDATE_1_2_5 "ALTER TABLE `mt_autoscan` CHANGE `location` `location` BLOB NULL DEFAULT NULL"
-#define MYSQL_UPDATE_1_2_6 "UPDATE `mt_internal_setting` SET `value`='2' WHERE `key`='db_version'"
 
 // updates 2->3
 #define MYSQL_UPDATE_2_3_1 "ALTER TABLE `mt_autoscan` CHANGE `scan_mode` `scan_mode` ENUM( 'timed', 'inotify' ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL"
 #define MYSQL_UPDATE_2_3_2 "ALTER TABLE `mt_autoscan` DROP INDEX `mt_autoscan_obj_id`, ADD UNIQUE `mt_autoscan_obj_id` ( `obj_id` )"
 #define MYSQL_UPDATE_2_3_3 "ALTER TABLE `mt_autoscan` ADD `path_ids` BLOB AFTER `location`"
-#define MYSQL_UPDATE_2_3_4 "UPDATE `mt_internal_setting` SET `value`='3' WHERE `key`='db_version' AND `value`='2'"
 
 // updates 3->4
 #define MYSQL_UPDATE_3_4_1 "ALTER TABLE `mt_cds_object` ADD `service_id` varchar(255) default NULL"
 #define MYSQL_UPDATE_3_4_2 "ALTER TABLE `mt_cds_object` ADD KEY `cds_object_service_id` (`service_id`)"
-#define MYSQL_UPDATE_3_4_3 "UPDATE `mt_internal_setting` SET `value`='4' WHERE `key`='db_version' AND `value`='3'"
 
 // updates 4->5
 #define MYSQL_UPDATE_4_5_1 "CREATE TABLE `mt_metadata` ( \
@@ -73,7 +70,6 @@
   KEY `metadata_item_id` (`item_id`), \
   CONSTRAINT `mt_metadata_idfk1` FOREIGN KEY (`item_id`) REFERENCES `mt_cds_object` (`id`) ON DELETE CASCADE ON UPDATE CASCADE \
 ) ENGINE=MyISAM CHARSET=utf8"
-#define MYSQL_UPDATE_4_5_2 "UPDATE `mt_internal_setting` SET `value`='5' WHERE `key`='db_version' AND `value`='4'"
 
 // updates 5->6: add config value table
 #define MYSQL_UPDATE_5_6_1 "CREATE TABLE `grb_config_value` ( \
@@ -83,29 +79,47 @@
   `status` varchar(20) NOT NULL) \
   ENGINE=MyISAM CHARSET=utf8"
 #define MYSQL_UPDATE_5_6_2 "CREATE INDEX grb_config_value_item ON grb_config_value(item)"
-#define MYSQL_UPDATE_5_6_3 "UPDATE `mt_internal_setting` SET `value`='6' WHERE `key`='db_version' AND `value`='5'"
 
 // updates 6->7
 #define MYSQL_UPDATE_6_7_1 "DROP TABLE mt_cds_active_item;"
-#define MYSQL_UPDATE_6_7_2 "UPDATE `mt_internal_setting` SET `value`='7' WHERE `key`='db_version' AND `value`='6'"
 
 // updates 7->8: part_number
 #define MYSQL_UPDATE_7_8_1 "ALTER TABLE `mt_cds_object` ADD `part_number` int(11) default NULL AFTER `flags`"
 #define MYSQL_UPDATE_7_8_2 "ALTER TABLE `mt_cds_object` DROP KEY `cds_object_track_number`"
 #define MYSQL_UPDATE_7_8_3 "ALTER TABLE `mt_cds_object` ADD KEY `cds_object_track_number` (`part_number`,`track_number`)"
-#define MYSQL_UPDATE_7_8_4 "UPDATE `mt_internal_setting` SET `value`='8' WHERE `key`='db_version' AND `value`='7'"
+
+// updates 8->9: bookmark_pos
+#define MYSQL_UPDATE_8_9_1 "ALTER TABLE `mt_cds_object` ADD `bookmark_pos` int(11) unsigned NOT NULL default '0' AFTER `service_id`"
+
+// updates 9->10: last_modified
+#define MYSQL_UPDATE_9_10_1 "ALTER TABLE `mt_cds_object` ADD `last_modified` bigint(20) unsigned default NULL AFTER `bookmark_pos`"
+
+#define MYSQL_UPDATE_VERSION "UPDATE `mt_internal_setting` SET `value`='{}' WHERE `key`='db_version' AND `value`='{}'"
+
+static const auto dbUpdates = std::array<std::vector<const char*>, 9> { {
+    { MYSQL_UPDATE_1_2_1, MYSQL_UPDATE_1_2_2, MYSQL_UPDATE_1_2_3, MYSQL_UPDATE_1_2_4, MYSQL_UPDATE_1_2_5 },
+    { MYSQL_UPDATE_2_3_1, MYSQL_UPDATE_2_3_2, MYSQL_UPDATE_2_3_3 },
+    { MYSQL_UPDATE_3_4_1, MYSQL_UPDATE_3_4_2 },
+    { MYSQL_UPDATE_4_5_1 },
+    { MYSQL_UPDATE_5_6_1, MYSQL_UPDATE_5_6_2 },
+    { MYSQL_UPDATE_6_7_1 },
+    { MYSQL_UPDATE_7_8_1, MYSQL_UPDATE_7_8_2, MYSQL_UPDATE_7_8_3 },
+    { MYSQL_UPDATE_8_9_1 },
+    { MYSQL_UPDATE_9_10_1 },
+} };
 
 MySQLDatabase::MySQLDatabase(std::shared_ptr<Config> config)
     : SQLDatabase(std::move(config))
+    , mysql_connection(false)
+    , mysql_init_key_initialized(false)
 {
-    mysql_init_key_initialized = false;
-    mysql_connection = false;
     table_quote_begin = '`';
     table_quote_end = '`';
 }
+
 MySQLDatabase::~MySQLDatabase()
 {
-    AutoLock lock(mysqlMutex); // just to ensure, that we don't close while another thread
+    SqlAutoLock lock(sqlMutex); // just to ensure, that we don't close while another thread
     // is executing a query
 
     if (mysql_connection) {
@@ -141,33 +155,23 @@ void MySQLDatabase::init()
     log_debug("start");
     SQLDatabase::init();
 
-    std::unique_lock<decltype(mysqlMutex)> lock(mysqlMutex);
-    int ret;
+    std::unique_lock<decltype(sqlMutex)> lock(sqlMutex);
 
     if (!mysql_thread_safe()) {
         throw_std_runtime_error("mysql library is not thread safe");
     }
 
     /// \todo write destructor function
-    ret = pthread_key_create(&mysql_init_key, nullptr);
+    int ret = pthread_key_create(&mysql_init_key, nullptr);
     if (ret) {
         throw_std_runtime_error("could not create pthread_key");
     }
     mysql_server_init(0, nullptr, nullptr);
     pthread_setspecific(mysql_init_key, reinterpret_cast<void*>(1));
 
-    std::string dbHost = config->getOption(CFG_SERVER_STORAGE_MYSQL_HOST);
-    std::string dbName = config->getOption(CFG_SERVER_STORAGE_MYSQL_DATABASE);
-    std::string dbUser = config->getOption(CFG_SERVER_STORAGE_MYSQL_USERNAME);
-    auto dbPort = in_port_t(config->getIntOption(CFG_SERVER_STORAGE_MYSQL_PORT));
-    std::string dbPass = config->getOption(CFG_SERVER_STORAGE_MYSQL_PASSWORD);
-    std::string dbSock = config->getOption(CFG_SERVER_STORAGE_MYSQL_SOCKET);
-
-    MYSQL* res_mysql;
-
-    res_mysql = mysql_init(&db);
+    MYSQL* res_mysql = mysql_init(&db);
     if (!res_mysql) {
-        throw_std_runtime_error("mysql_init failed");
+        throw_std_runtime_error("mysql_init() failed");
     }
 
     mysql_init_key_initialized = true;
@@ -177,6 +181,13 @@ void MySQLDatabase::init()
     bool my_bool_var = true;
     mysql_options(&db, MYSQL_OPT_RECONNECT, &my_bool_var);
 
+    std::string dbHost = config->getOption(CFG_SERVER_STORAGE_MYSQL_HOST);
+    std::string dbName = config->getOption(CFG_SERVER_STORAGE_MYSQL_DATABASE);
+    std::string dbUser = config->getOption(CFG_SERVER_STORAGE_MYSQL_USERNAME);
+    auto dbPort = in_port_t(config->getIntOption(CFG_SERVER_STORAGE_MYSQL_PORT));
+    std::string dbPass = config->getOption(CFG_SERVER_STORAGE_MYSQL_PASSWORD);
+    std::string dbSock = config->getOption(CFG_SERVER_STORAGE_MYSQL_SOCKET);
+
     res_mysql = mysql_real_connect(&db,
         dbHost.c_str(),
         dbUser.c_str(),
@@ -187,7 +198,7 @@ void MySQLDatabase::init()
         0 // flags
     );
     if (!res_mysql) {
-        throw_std_runtime_error("The connection to the MySQL database has failed: {}", getError(&db));
+        throw_std_runtime_error("Connecting to database {}:{}/{} failed: {}", dbHost, dbPort, dbName, getError(&db));
     }
 
     mysql_connection = true;
@@ -205,7 +216,12 @@ void MySQLDatabase::init()
         log_debug("Loading initialisation SQL from: {}", sqlFilePath.c_str());
         auto sql = readTextFile(sqlFilePath);
 
-        for (const auto& statement : splitString(sql, ';')) {
+        for (auto&& statement : splitString(sql, ';')) {
+            trimStringInPlace(statement);
+            if (statement.empty()) {
+                continue;
+            }
+            log_debug("executing statement: '{}'", statement);
             ret = mysql_real_query(&db, statement.c_str(), statement.size());
             if (ret) {
                 std::string myError = getError(&db);
@@ -223,73 +239,21 @@ void MySQLDatabase::init()
     log_debug("db_version: {}", dbVersion.c_str());
 
     /* --- database upgrades --- */
-    if (dbVersion == "1") {
-        log_info("Doing an automatic database upgrade from database version 1 to version 2...");
-        _exec(MYSQL_UPDATE_1_2_1);
-        _exec(MYSQL_UPDATE_1_2_2);
-        _exec(MYSQL_UPDATE_1_2_3);
-        _exec(MYSQL_UPDATE_1_2_4);
-        _exec(MYSQL_UPDATE_1_2_5);
-        _exec(MYSQL_UPDATE_1_2_6);
-        log_info("database upgrade successful.");
-        dbVersion = "2";
+    int version = 1;
+    for (auto&& upgrade : dbUpdates) {
+        if (dbVersion == fmt::to_string(version)) {
+            log_info("Running an automatic database upgrade from database version {} to version {}...", version, version + 1);
+            for (auto&& upgradeCmd : upgrade) {
+                _exec(upgradeCmd);
+            }
+            _exec(fmt::format(MYSQL_UPDATE_VERSION, version + 1, version).c_str());
+            dbVersion = fmt::to_string(version + 1);
+            log_info("Database upgrade to version {} successful.", dbVersion.c_str());
+        }
+        version++;
     }
 
-    if (dbVersion == "2") {
-        log_info("Doing an automatic database upgrade from database version 2 to version 3...");
-        _exec(MYSQL_UPDATE_2_3_1);
-        _exec(MYSQL_UPDATE_2_3_2);
-        _exec(MYSQL_UPDATE_2_3_3);
-        _exec(MYSQL_UPDATE_2_3_4);
-        log_info("database upgrade successful.");
-        dbVersion = "3";
-    }
-
-    if (dbVersion == "3") {
-        log_info("Doing an automatic database upgrade from database version 3 to version 4...");
-        _exec(MYSQL_UPDATE_3_4_1);
-        _exec(MYSQL_UPDATE_3_4_2);
-        _exec(MYSQL_UPDATE_3_4_3);
-        log_info("database upgrade successful.");
-        dbVersion = "4";
-    }
-
-    if (dbVersion == "4") {
-        log_info("Doing an automatic database upgrade from database version 4 to version 5...");
-        _exec(MYSQL_UPDATE_4_5_1);
-        _exec(MYSQL_UPDATE_4_5_2);
-        log_info("database upgrade successful.");
-        dbVersion = "5";
-    }
-
-    if (dbVersion == "5") {
-        log_info("Doing an automatic database upgrade from database version 5 to version 6...");
-        _exec(MYSQL_UPDATE_5_6_1);
-        _exec(MYSQL_UPDATE_5_6_2);
-        _exec(MYSQL_UPDATE_5_6_3);
-        log_info("database upgrade successful.");
-        dbVersion = "6";
-    }
-
-    if (dbVersion == "6") {
-        log_info("Doing an automatic database upgrade from database version 6 to version 7...");
-        _exec(MYSQL_UPDATE_6_7_1);
-        _exec(MYSQL_UPDATE_6_7_2);
-        log_info("database upgrade successful.");
-        dbVersion = "7";
-    }
-
-    if (dbVersion == "7") {
-        log_info("Doing an automatic database upgrade from database version 7 to version 8...");
-        _exec(MYSQL_UPDATE_7_8_1);
-        _exec(MYSQL_UPDATE_7_8_2);
-        _exec(MYSQL_UPDATE_7_8_3);
-        _exec(MYSQL_UPDATE_7_8_4);
-        log_info("database upgrade successful.");
-        dbVersion = "8";
-    }
-
-    if (dbVersion != "8")
+    if (dbVersion != fmt::to_string(version))
         throw_std_runtime_error("The database seems to be from a newer version (database version {})", dbVersion);
 
     lock.unlock();
@@ -311,7 +275,7 @@ std::string MySQLDatabase::quote(std::string value) const
      */
     auto q = new char[value.length() * 2 + 2];
     *q = '\'';
-    long size = mysql_real_escape_string(const_cast<MYSQL*>(&db), q + 1, value.c_str(), value.length());
+    auto size = mysql_real_escape_string(const_cast<MYSQL*>(&db), q + 1, value.c_str(), value.length());
     q[size + 1] = '\'';
     std::string ret(q, size + 2);
     delete[] q;
@@ -320,11 +284,42 @@ std::string MySQLDatabase::quote(std::string value) const
 
 std::string MySQLDatabase::getError(MYSQL* db)
 {
-    std::ostringstream err_buf;
-    err_buf << "mysql_error (" << mysql_errno(db);
-    err_buf << "): \"" << mysql_error(db) << "\"";
-    log_debug("{}", err_buf.str().c_str());
-    return err_buf.str();
+    auto res = fmt::format("mysql_error ({}): \"{}\"", mysql_errno(db), mysql_error(db));
+    log_debug("{}", res);
+    return res;
+}
+
+void MySQLDatabase::beginTransaction(const std::string_view& tName)
+{
+    log_debug("START TRANSACTION {} {}", tName, inTransaction);
+    StdThreadRunner::waitFor(
+        "MySqlDatabase", [this] { return !inTransaction; }, 100);
+    inTransaction = true;
+    log_debug("START TRANSACTION {}", tName);
+    SqlAutoLock lock(sqlMutex);
+    if (use_transaction)
+        _exec("START TRANSACTION");
+}
+
+void MySQLDatabase::rollback(const std::string_view& tName)
+{
+    log_debug("ROLLBACK {}", tName);
+    if (use_transaction && inTransaction && mysql_rollback(&db)) {
+        std::string myError = getError(&db);
+        throw DatabaseException(myError, fmt::format("Mysql: error while rolling back db: {}", myError));
+    }
+    inTransaction = false;
+}
+
+void MySQLDatabase::commit(const std::string_view& tName)
+{
+    log_debug("COMMIT {}", tName);
+    SqlAutoLock lock(sqlMutex);
+    if (use_transaction && inTransaction && mysql_commit(&db)) {
+        std::string myError = getError(&db);
+        throw DatabaseException(myError, fmt::format("Mysql: error while commiting db: {}", myError));
+    }
+    inTransaction = false;
 }
 
 std::shared_ptr<SQLResult> MySQLDatabase::select(const char* query, int length)
@@ -334,21 +329,28 @@ std::shared_ptr<SQLResult> MySQLDatabase::select(const char* query, int length)
     print_backtrace();
 #endif
 
-    int res;
-
     checkMysqlThreadInit();
-    AutoLock lock(mysqlMutex);
-    res = mysql_real_query(&db, query, length);
+    SqlAutoLock lock(sqlMutex);
+    bool myTransaction = false;
+    if (!inTransaction) { // protect calls outside transactions
+        inTransaction = true;
+        myTransaction = true;
+    }
+    auto res = mysql_real_query(&db, query, length);
     if (res) {
         std::string myError = getError(&db);
-        throw DatabaseException(myError, "Mysql: mysql_real_query() failed: " + myError + "; query: " + query);
+        rollback("");
+        throw DatabaseException(myError, fmt::format("Mysql: mysql_real_query() failed: {}; query: {}", myError, query));
     }
 
-    MYSQL_RES* mysql_res;
-    mysql_res = mysql_store_result(&db);
-    if (!mysql_res) {
+    MYSQL_RES* mysql_res = mysql_store_result(&db);
+    if (!mysql_res && mysql_field_count(&db)) {
         std::string myError = getError(&db);
-        throw DatabaseException(myError, "Mysql: mysql_store_result() failed: " + myError + "; query: " + query);
+        rollback("");
+        throw DatabaseException(myError, fmt::format("Mysql: mysql_store_result() failed: {}; query: {}", myError, query));
+    }
+    if (myTransaction) {
+        inTransaction = false;
     }
 
     return std::static_pointer_cast<SQLResult>(std::make_shared<MysqlResult>(mysql_res));
@@ -361,14 +363,13 @@ int MySQLDatabase::exec(const char* query, int length, bool getLastInsertId)
     print_backtrace();
 #endif
 
-    int res;
-
     checkMysqlThreadInit();
-    AutoLock lock(mysqlMutex);
-    res = mysql_real_query(&db, query, length);
+    SqlAutoLock lock(sqlMutex);
+    auto res = mysql_real_query(&db, query, length);
     if (res) {
         std::string myError = getError(&db);
-        throw DatabaseException(myError, "Mysql: mysql_real_query() failed: " + myError + "; query: " + query);
+        rollback("");
+        throw DatabaseException(myError, fmt::format("Mysql: mysql_real_query() failed: {}; query: {}", myError, query));
     }
     int insert_id = -1;
     if (getLastInsertId)
@@ -389,31 +390,30 @@ void MySQLDatabase::storeInternalSetting(const std::string& key, const std::stri
       << quote(key) << ", " << quotedValue << ") "
                                               "ON DUPLICATE KEY UPDATE `value` = "
       << quotedValue;
-    SQLDatabase::exec(q);
+    SQLDatabase::exec(q.str());
 }
 
 void MySQLDatabase::_exec(const char* query, int length)
 {
     if (mysql_real_query(&db, query, (length > 0 ? length : strlen(query)))) {
         std::string myError = getError(&db);
-        throw DatabaseException(myError, "Mysql: error while updating db: " + myError);
+        throw DatabaseException(myError, fmt::format("Mysql: error while updating db: {}; query: {}", myError, query));
     }
 }
 
 /* MysqlResult */
 
 MysqlResult::MysqlResult(MYSQL_RES* mysql_res)
+    : nullRead(false)
+    , mysql_res(mysql_res)
 {
-    this->mysql_res = mysql_res;
-    nullRead = false;
 }
 
 MysqlResult::~MysqlResult()
 {
     if (mysql_res) {
         if (!nullRead) {
-            MYSQL_ROW mysql_row;
-            while ((mysql_row = mysql_fetch_row(mysql_res)) != nullptr)
+            while ((mysql_fetch_row(mysql_res)) != nullptr)
                 ; // read out data
         }
         mysql_free_result(mysql_res);
@@ -423,11 +423,8 @@ MysqlResult::~MysqlResult()
 
 std::unique_ptr<SQLRow> MysqlResult::nextRow()
 {
-    MYSQL_ROW mysql_row;
-    mysql_row = mysql_fetch_row(mysql_res);
-    if (mysql_row) {
-        auto p = std::make_unique<MysqlRow>(mysql_row);
-        return p;
+    if (auto mysql_row = mysql_fetch_row(mysql_res); mysql_row != nullptr) {
+        return std::make_unique<MysqlRow>(mysql_row);
     }
     nullRead = true;
     mysql_free_result(mysql_res);
@@ -438,8 +435,8 @@ std::unique_ptr<SQLRow> MysqlResult::nextRow()
 /* MysqlRow */
 
 MysqlRow::MysqlRow(MYSQL_ROW mysql_row)
+    : mysql_row(mysql_row)
 {
-    this->mysql_row = mysql_row;
 }
 
 #endif // HAVE_MYSQL
