@@ -32,11 +32,11 @@
 #include "generic_task.h" // API
 
 GenericTask::GenericTask(task_owner_t taskOwner)
+    : taskType(Invalid)
+    , taskOwner(taskOwner)
+    , parentTaskID(0)
+    , taskID(0)
+    , valid(true)
+    , cancellable(true)
 {
-    valid = true;
-    cancellable = true;
-    taskType = Invalid;
-    taskID = 0;
-    parentTaskID = 0;
-    this->taskOwner = taskOwner;
 }
