@@ -29,6 +29,7 @@
 #include <numeric>
 
 #include "config/client_config.h"
+#include "config/config_definition.h"
 #include "config/config_setup.h"
 #include "config/directory_tweak.h"
 #include "content/autoscan.h"
@@ -63,16 +64,21 @@ void web::configLoad::addTypeMeta(pugi::xml_node& meta, const std::shared_ptr<Co
     info.append_attribute("help") = cs->getHelp();
 }
 
-void web::configLoad::createItem(pugi::xml_node& item, const std::string& name, config_option_t id, config_option_t aid)
+void web::configLoad::createItem(pugi::xml_node& item, const std::string& name, config_option_t id, config_option_t aid, const std::shared_ptr<ConfigSetup>& cs)
 {
     allItems[name] = &item;
     item.append_attribute("item") = name.c_str();
     item.append_attribute("id") = fmt::format("{:02d}", id).c_str();
     item.append_attribute("aid") = fmt::format("{:02d}", aid).c_str();
-    item.append_attribute("status") = "unchanged";
-
+    if (std::any_of(dbEntries.begin(), dbEntries.end(), [&](auto&& s) { return s.item == name; })) {
+        item.append_attribute("status") = "unchanged";
+        item.append_attribute("source") = "database";
+    } else {
+        item.append_attribute("status") = cs == nullptr || !cs->isDefaultValueUsed() ? "unchanged" : "default";
+        item.append_attribute("source") = cs == nullptr || !cs->isDefaultValueUsed() ? "config.xml" : "default";
+    }
     item.append_attribute("origValue") = config->getOrigValue(name).c_str();
-    item.append_attribute("source") = std::any_of(dbEntries.begin(), dbEntries.end(), [&](const auto& s) { return s.item == name; }) ? "database" : "config.xml";
+    item.append_attribute("defaultValue") = cs != nullptr ? cs->getDefaultValue().c_str() : "";
 }
 
 template <typename T>
@@ -94,12 +100,14 @@ void web::configLoad::setValue(pugi::xml_node& item, const fs::path& value)
     item.append_attribute("value") = value.c_str();
 }
 
+/// \brief: process config_load request
 void web::configLoad::process()
 {
     check_request();
     auto root = xmlDoc->document_element();
     auto values = root.append_child("values");
 
+    // set handling of json properties
     xml2JsonHints->setArrayName(values, "item");
     xml2JsonHints->setFieldType("item", "string");
     xml2JsonHints->setFieldType("id", "string");
@@ -109,12 +117,14 @@ void web::configLoad::process()
 
     log_debug("Sending Config to web!");
 
+    // generate meta info for ui
     auto meta = root.append_child("types");
     xml2JsonHints->setArrayName(meta, "item");
-    for (const auto& cs : ConfigManager::getOptionList()) {
+    for (auto&& cs : ConfigDefinition::getOptionList()) {
         addTypeMeta(meta, cs);
     }
 
+    // write database status
     {
         auto item = values.append_child("item");
         createItem(item, "/status/attribute::total", CFG_MAX, CFG_MAX);
@@ -144,10 +154,11 @@ void web::configLoad::process()
         setValue(item, database->getTotalFiles(true, "image"));
     }
 
+    // write all values with simple type (string, int, bool)
     for (int i = 0; i < int(CFG_MAX); i++) {
-        auto scs = ConfigManager::findConfigSetup(config_option_t(i));
+        auto scs = ConfigDefinition::findConfigSetup(config_option_t(i));
         auto item = values.append_child("item");
-        createItem(item, scs->getItemPath(-1), config_option_t(i), config_option_t(i));
+        createItem(item, scs->getItemPath(-1), config_option_t(i), config_option_t(i), scs);
 
         try {
             log_debug("    Option {:03d} {} = {}", i, scs->getItemPath(), scs->getCurrentValue().c_str());
@@ -156,26 +167,28 @@ void web::configLoad::process()
         }
     }
 
+    // write client configuration
     std::shared_ptr<ConfigSetup> cs;
-    cs = ConfigManager::findConfigSetup(CFG_CLIENTS_LIST);
+    cs = ConfigDefinition::findConfigSetup(CFG_CLIENTS_LIST);
     auto clientConfig = cs->getValue()->getClientConfigListOption();
     for (size_t i = 0; i < clientConfig->size(); i++) {
         auto client = clientConfig->get(i);
 
         auto item = values.append_child("item");
-        createItem(item, cs->getItemPath(i, ATTR_CLIENTS_CLIENT_FLAGS), cs->option, ATTR_CLIENTS_CLIENT_FLAGS);
+        createItem(item, cs->getItemPath(i, ATTR_CLIENTS_CLIENT_FLAGS), cs->option, ATTR_CLIENTS_CLIENT_FLAGS, cs);
         setValue(item, ClientConfig::mapFlags(client->getFlags()));
 
         item = values.append_child("item");
-        createItem(item, cs->getItemPath(i, ATTR_CLIENTS_CLIENT_IP), cs->option, ATTR_CLIENTS_CLIENT_IP);
+        createItem(item, cs->getItemPath(i, ATTR_CLIENTS_CLIENT_IP), cs->option, ATTR_CLIENTS_CLIENT_IP, cs);
         setValue(item, client->getIp());
 
         item = values.append_child("item");
-        createItem(item, cs->getItemPath(i, ATTR_CLIENTS_CLIENT_USERAGENT), cs->option, ATTR_CLIENTS_CLIENT_USERAGENT);
+        createItem(item, cs->getItemPath(i, ATTR_CLIENTS_CLIENT_USERAGENT), cs->option, ATTR_CLIENTS_CLIENT_USERAGENT, cs);
         setValue(item, client->getUserAgent());
     }
 
-    cs = ConfigManager::findConfigSetup(CFG_IMPORT_DIRECTORIES_LIST);
+    // write import tweaks
+    cs = ConfigDefinition::findConfigSetup(CFG_IMPORT_DIRECTORIES_LIST);
     auto directoryConfig = cs->getValue()->getDirectoryTweakOption();
     for (size_t i = 0; i < directoryConfig->size(); i++) {
         auto dir = directoryConfig->get(i);
@@ -221,18 +234,19 @@ void web::configLoad::process()
         setValue(item, dir->hasSubTitleFile() ? dir->getSubTitleFile() : "");
     }
 
-    cs = ConfigManager::findConfigSetup(CFG_TRANSCODING_PROFILE_LIST);
+    // write transconding configuration
+    cs = ConfigDefinition::findConfigSetup(CFG_TRANSCODING_PROFILE_LIST);
     auto transcoding = cs->getValue()->getTranscodingProfileListOption();
     int pr = 0;
     std::map<std::string, int> profiles;
-    for (const auto& [key, val] : transcoding->getList()) {
-        for (const auto& [a, name] : *val) {
+    for (auto&& [key, val] : transcoding->getList()) {
+        for (auto&& [a, name] : *val) {
             auto item = values.append_child("item");
-            createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_MIMETYPE_PROF_MAP, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_TRANSCODE, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_MIMETYPE), cs->option, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_MIMETYPE);
+            createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_MIMETYPE_PROF_MAP, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_TRANSCODE, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_MIMETYPE), cs->option, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_MIMETYPE, cs);
             setValue(item, key);
 
             item = values.append_child("item");
-            createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_MIMETYPE_PROF_MAP, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_TRANSCODE, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_USING), cs->option, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_USING);
+            createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_MIMETYPE_PROF_MAP, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_TRANSCODE, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_USING), cs->option, ATTR_TRANSCODING_MIMETYPE_PROF_MAP_USING, cs);
             setValue(item, name->getName());
             profiles[name->getName()] = pr;
 
@@ -241,7 +255,7 @@ void web::configLoad::process()
     }
 
     pr = 0;
-    for (const auto& [key, val] : profiles) {
+    for (auto&& [key, val] : profiles) {
         auto entry = transcoding->getByName(key, true);
         auto item = values.append_child("item");
         createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_NAME), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_NAME);
@@ -291,10 +305,6 @@ void web::configLoad::process()
         createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_ACCOGG), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_ACCOGG);
         setValue(item, entry->isTheora());
 
-        item = values.append_child("item");
-        createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_USECHUNKEDENC), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_USECHUNKEDENC);
-        setValue(item, entry->getChunked());
-
         item = values.append_child("item");
         createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_AGENT, ATTR_TRANSCODING_PROFILES_PROFLE_AGENT_COMMAND), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_AGENT_COMMAND);
         setValue(item, entry->getCommand());
@@ -325,126 +335,89 @@ void web::configLoad::process()
             if (!fourCCList.empty()) {
                 item = values.append_child("item");
                 createItem(item, cs->getItemPath(pr, ATTR_TRANSCODING_PROFILES_PROFLE, ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC, ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_4CC), cs->option, ATTR_TRANSCODING_PROFILES_PROFLE_AVI4CC_4CC);
-                setValue(item, std::accumulate(std::next(fourCCList.begin()), fourCCList.end(), fourCCList[0], [](const std::string& a, const std::string& b) { return fmt::format("{}, {}", a.c_str(), b.c_str()); }));
+                setValue(item, std::accumulate(std::next(fourCCList.begin()), fourCCList.end(), fourCCList[0], [](auto&& a, auto&& b) { return fmt::format("{}, {}", a.c_str(), b.c_str()); }));
             }
         }
         pr++;
     }
 
-#ifdef HAVE_INOTIFY
-    constexpr auto autoscanList = std::array { CFG_IMPORT_AUTOSCAN_TIMED_LIST, CFG_IMPORT_AUTOSCAN_INOTIFY_LIST };
-#else
-    constexpr auto autoscanList = std::array { CFG_IMPORT_AUTOSCAN_TIMED_LIST };
-#endif
-    for (const auto& autoscanOption : autoscanList) {
-        cs = ConfigManager::findConfigSetup(autoscanOption);
-        auto autoscan = cs->getValue()->getAutoscanListOption();
+    // write autoscan configuration
+    for (auto&& ascs : ConfigDefinition::getConfigSetupList<ConfigAutoscanSetup>()) {
+        auto autoscan = ascs->getValue()->getAutoscanListOption();
         for (size_t i = 0; i < autoscan->size(); i++) {
-            const auto& entry = autoscan->get(i);
-            const auto& adir = content->getAutoscanDirectory(entry->getLocation());
+            auto&& entry = autoscan->get(i);
+            auto&& adir = content->getAutoscanDirectory(entry->getLocation());
             auto item = values.append_child("item");
-            createItem(item, cs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_LOCATION), cs->option, ATTR_AUTOSCAN_DIRECTORY_LOCATION);
+            createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_LOCATION), ascs->option, ATTR_AUTOSCAN_DIRECTORY_LOCATION);
             setValue(item, adir->getLocation());
 
             item = values.append_child("item");
-            createItem(item, cs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_MODE), cs->option, ATTR_AUTOSCAN_DIRECTORY_MODE);
+            createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_MODE), ascs->option, ATTR_AUTOSCAN_DIRECTORY_MODE);
             setValue(item, AutoscanDirectory::mapScanmode(adir->getScanMode()));
 
             item = values.append_child("item");
-            createItem(item, cs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_INTERVAL), cs->option, ATTR_AUTOSCAN_DIRECTORY_INTERVAL);
+            createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_INTERVAL), ascs->option, ATTR_AUTOSCAN_DIRECTORY_INTERVAL);
             setValue(item, adir->getInterval());
 
             item = values.append_child("item");
-            createItem(item, cs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_RECURSIVE), cs->option, ATTR_AUTOSCAN_DIRECTORY_RECURSIVE);
+            createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_RECURSIVE), ascs->option, ATTR_AUTOSCAN_DIRECTORY_RECURSIVE);
             setValue(item, adir->getRecursive());
 
             item = values.append_child("item");
-            createItem(item, cs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_HIDDENFILES), cs->option, ATTR_AUTOSCAN_DIRECTORY_HIDDENFILES);
+            createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_HIDDENFILES), ascs->option, ATTR_AUTOSCAN_DIRECTORY_HIDDENFILES);
             setValue(item, adir->getHidden());
 
             item = values.append_child("item");
-            createItem(item, cs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_SCANCOUNT), cs->option, ATTR_AUTOSCAN_DIRECTORY_SCANCOUNT);
+            createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_SCANCOUNT), ascs->option, ATTR_AUTOSCAN_DIRECTORY_SCANCOUNT);
             setValue(item, adir->getActiveScanCount());
 
             item = values.append_child("item");
-            createItem(item, cs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_TASKCOUNT), cs->option, ATTR_AUTOSCAN_DIRECTORY_TASKCOUNT);
+            createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_TASKCOUNT), ascs->option, ATTR_AUTOSCAN_DIRECTORY_TASKCOUNT);
             setValue(item, adir->getTaskCount());
 
             item = values.append_child("item");
-            createItem(item, cs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_LMT), cs->option, ATTR_AUTOSCAN_DIRECTORY_LMT);
-            setValue(item, fmt::format("{:%Y-%m-%d %H:%M:%S}", fmt::localtime(adir->getPreviousLMT(""))));
+            createItem(item, ascs->getItemPath(i, ATTR_AUTOSCAN_DIRECTORY_LMT), ascs->option, ATTR_AUTOSCAN_DIRECTORY_LMT);
+            setValue(item, fmt::format("{:%Y-%m-%d %H:%M:%S}", fmt::localtime(adir->getPreviousLMT().count())));
         }
     }
 
-    constexpr auto dict_options = std::array {
-        CFG_SERVER_UI_ACCOUNT_LIST,
-        CFG_IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_LIST,
-        CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST,
-        CFG_IMPORT_MAPPINGS_MIMETYPE_TO_UPNP_CLASS_LIST,
-        CFG_IMPORT_LAYOUT_MAPPING,
-        CFG_IMPORT_SCRIPTING_IMPORT_GENRE_MAP,
-#ifdef HAVE_JS
-        CFG_IMPORT_SCRIPTING_IMPORT_SCRIPT_OPTIONS,
-#endif
-    };
-
-    for (const auto& dict_option : dict_options) {
+    // write content of all dictionaries
+    for (auto&& dcs : ConfigDefinition::getConfigSetupList<ConfigDictionarySetup>()) {
         int i = 0;
-        auto dcs = ConfigSetup::findConfigSetup<ConfigDictionarySetup>(dict_option);
         auto dictionary = dcs->getValue()->getDictionaryOption(true);
-        for (const auto& [key, val] : dictionary) {
+        for (auto&& [key, val] : dictionary) {
             auto item = values.append_child("item");
-            createItem(item, dcs->getItemPath(i, dcs->keyOption), dcs->option, dcs->keyOption);
+            createItem(item, dcs->getItemPath(i, dcs->keyOption), dcs->option, dcs->keyOption, dcs);
             setValue(item, key.substr(5));
 
             item = values.append_child("item");
-            createItem(item, dcs->getItemPath(i, dcs->valOption), dcs->option, dcs->valOption);
+            createItem(item, dcs->getItemPath(i, dcs->valOption), dcs->option, dcs->valOption, dcs);
             setValue(item, val);
             i++;
         }
     }
 
-    constexpr auto array_options = std::array {
-        CFG_SERVER_UI_ITEMS_PER_PAGE_DROPDOWN,
-        CFG_IMPORT_RESOURCES_FANART_FILE_LIST,
-        CFG_IMPORT_RESOURCES_CONTAINERART_FILE_LIST,
-        CFG_IMPORT_RESOURCES_SUBTITLE_FILE_LIST,
-        CFG_IMPORT_RESOURCES_RESOURCE_FILE_LIST,
-        CFG_SERVER_EXTOPTS_MARK_PLAYED_ITEMS_CONTENT_LIST,
-#ifdef HAVE_LIBEXIF
-        CFG_IMPORT_LIBOPTS_EXIF_AUXDATA_TAGS_LIST,
-#endif
-#ifdef HAVE_EXIV2
-        CFG_IMPORT_LIBOPTS_EXIV2_AUXDATA_TAGS_LIST,
-#endif
-#ifdef HAVE_TAGLIB
-        CFG_IMPORT_LIBOPTS_ID3_AUXDATA_TAGS_LIST,
-#endif
-#ifdef HAVE_FFMPEG
-        CFG_IMPORT_LIBOPTS_FFMPEG_AUXDATA_TAGS_LIST,
-#endif
-    };
-
-    for (auto array_option : array_options) {
-        auto acs = ConfigSetup::findConfigSetup<ConfigArraySetup>(array_option);
+    // write content of all arrays
+    for (auto&& acs : ConfigDefinition::getConfigSetupList<ConfigArraySetup>()) {
         auto array = acs->getValue()->getArrayOption(true);
         for (size_t i = 0; i < array.size(); i++) {
             auto entry = array[i];
             auto item = values.append_child("item");
-            createItem(item, acs->getItemPath(i), acs->option, acs->attrOption != CFG_MAX ? acs->attrOption : acs->nodeOption);
+            createItem(item, acs->getItemPath(i), acs->option, acs->attrOption != CFG_MAX ? acs->attrOption : acs->nodeOption, acs);
             setValue(item, entry);
         }
     }
 
-    for (const auto& entry : dbEntries) {
+    // update entries with datebase values
+    for (auto&& entry : dbEntries) {
         auto exItem = allItems.find(entry.item);
         if (exItem != allItems.end()) {
             auto item = exItem->second;
             item->attribute("source") = "database";
             item->attribute("status") = entry.status.c_str();
         } else {
-            auto cs = ConfigManager::findConfigSetupByPath(entry.item, true);
-            auto acs = ConfigManager::findConfigSetupByPath(entry.item, true, cs);
+            auto cs = ConfigDefinition::findConfigSetupByPath(entry.item, true);
+            auto acs = ConfigDefinition::findConfigSetupByPath(entry.item, true, cs);
             if (cs != nullptr) {
                 auto item = values.append_child("item");
                 createItem(item, entry.item, cs->option, acs != nullptr ? acs->option : CFG_MAX);
