@@ -68,7 +68,10 @@ public:
     duk_ret_t dukMockPlaylist(duk_context* ctx, string title, string location, string mimetype);
 
     // Add global Duktape methods to proxy into c++ layer
-    void addGlobalFunctions(duk_context* ctx, const duk_function_list_entry* funcs);
+    void addGlobalFunctions(duk_context* ctx, const duk_function_list_entry* funcs, const std::map<std::string_view, std::string_view>& config = {});
+
+    // Add config entries to global context
+    static void addConfig(duk_context* ctx, const std::map<std::string_view, std::string_view>& config);
 
     // Access the global object(script) by name, and execute
     void executeScript(duk_context* ctx);
